package com.unitech.rfidreader.Listener;

import android.annotation.SuppressLint;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.os.AsyncTask;
import android.os.Handler;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.TableLayout;
import android.widget.TextView;
import android.widget.ToggleButton;

import com.thingmagic.Reader;
import com.unitech.ReaderConnect;
import com.unitech.rfidreader.R;
import com.unitech.rfidreader.ReaderActivity;
import com.unitech.util.LoggerUtil;

import java.io.File;

@SuppressLint("StaticFieldLeak")
public class ConnectionListener implements View.OnClickListener {

    private static ReaderActivity mReaderActivity;
    private static Reader reader = null;
    private static LinearLayout readcontrollayout;
    private static LinearLayout layout_firmware, firmware_hint;

    private static TableLayout table = null;
    private static TextView searchResultCount = null;
    private static TextView totalTagCountView = null;
    private static TextView avgReadView = null;
    private static TextView readingTimeView = null;
    private static TextView temperatureView = null;
    private static TextView errorMsgView = null;
    private static Button connectButton;
    private static Button readButton = null;
    private static ProgressDialog pDialog = null;

    private static String TAG = "ConnectionListener";
    private static String readerName = null;
    private static Spinner regionList = null;

    private static Spinner baudrateList = null;

    private static Spinner powerList = null;
    private static ToggleButton powerButton;

    private static Reader.Region region = Reader.Region.NA;
    private static Handler mHandler;
    private static String UART;
    private static boolean isHsUART = false;

    public ConnectionListener(ReaderActivity readerActivity) {
        mReaderActivity = readerActivity;
        mHandler = new Handler();
        pDialog = new ProgressDialog(readerActivity);
        pDialog.setCancelable(false);
        pDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        findAllViewsById();
    }

    private void findAllViewsById() {
        connectButton = (Button) mReaderActivity.findViewById(R.id.btn_connect);
        readcontrollayout = (LinearLayout) mReaderActivity.findViewById(R.id.ReadControlLayout);
        layout_firmware = mReaderActivity.firmware.findViewById(R.id.layout_firmware);
        firmware_hint = mReaderActivity.firmware.findViewById(R.id.firmware_hint);

        table = (TableLayout) mReaderActivity.findViewById(R.id.tablelayout);
        searchResultCount = (TextView) mReaderActivity.findViewById(R.id.search_result_view);
        totalTagCountView = (TextView) mReaderActivity.findViewById(R.id.totalTagCount_view);
        avgReadView = mReaderActivity.findViewById(R.id.avg_reads_view);
        errorMsgView = mReaderActivity.findViewById(R.id.error_messages_view);
        readingTimeView = mReaderActivity.findViewById(R.id.reading_time);
        temperatureView = mReaderActivity.findViewById(R.id.temperature);
        readButton = (Button) mReaderActivity.findViewById(R.id.btn_scan);
        regionList = (Spinner) mReaderActivity.findViewById(R.id.Region_List);

        baudrateList = mReaderActivity.readOptions.findViewById(R.id.spinner_baudrate);

        powerList = (Spinner) mReaderActivity.readOptions.findViewById(R.id.antennaPower);

        powerButton = mReaderActivity.findViewById(R.id.btn_power);
    }

    @Override
    public void onClick(View arg0) {
        searchResultCount.setText("");
        totalTagCountView.setText("");
        table.removeAllViews();

        if (connectButton.getText().toString().equalsIgnoreCase("Connect")) {
            readerName = "Reader";

            File devFolder = new File("/dev");
            if (devFolder.exists() && devFolder.isDirectory()) {
                if (new File("/dev/ttyHS3").exists()) {
//                    UART = "tmr:///dev/ttyHS3";
                    UART = "ute:///dev/ttyHS3";
                    isHsUART = true;
                } else if (new File("/dev/ttyMSM2").exists()) {
                    UART = "tmr:///dev/ttyMSM2";
                    isHsUART = false;
                }
            }
            System.out.println("UART :" + UART);

            region = Reader.Region.valueOf(regionList.getSelectedItem().toString());

            LoggerUtil.debug(TAG, "Selected Region :" + region);
//            }
        }

        ReaderConnectionThread readerConnectionThread = new ReaderConnectionThread(
                UART, connectButton.getText().toString());
        readerConnectionThread.execute();
    }

    private static class ReaderConnectionThread extends AsyncTask<Void, Void, String> {
        private static final int timeout = 20;
        private String uriString = "";
        private String operation;
        private boolean operationStatus = true;
        private static Runnable timeoutRunnable = null;

        ReaderConnectionThread(String requestedQuery, String operation) {
            this.uriString = requestedQuery;
            this.operation = operation;
        }

        @Override
        protected void onPreExecute() {
            LoggerUtil.debug(TAG, "** ReaderConnectionThread.onPreExecute **");
            if (operation.equalsIgnoreCase("Connect")) {
                disableEdit();
                pDialog.setMessage("Connecting. Please wait...");
            } else {
                readButton.setClickable(false);
                readButton.setEnabled(false);
                disableEdit();
                pDialog.setMessage("Disconnecting. Please wait...");
            }
            pDialog.show();

//            timerDelayRemoveDialog(timeout * 1000, pDialog);

            searchResultCount.setText("");
            avgReadView.setText("");
            readingTimeView.setText("");
            temperatureView.setText("");
            errorMsgView.setText("");
            errorMsgView.setVisibility(View.GONE);
            ServiceListener.clearTagRecords();
        }

        @Override
        protected String doInBackground(Void... params) {
            String exception = "Exception :";
            try {
                if (operation.equalsIgnoreCase("Connect")) {
                    LoggerUtil.debug(TAG, "Start connecting");

                    LoggerUtil.debug(TAG, "uriString=" + uriString);

                    reader = ReaderConnect.connect(uriString, region, isHsUART);

                    LoggerUtil.debug(TAG, "Reader Connected");

//                    ServiceListener.loadDefaultParams(reader);
                } else {
                    reader.destroy();
                    reader = null;
                    LoggerUtil.debug(TAG, "Reader Disconnected");
                }

            } catch (Exception ex) {
                operationStatus = false;
                if (ex.getMessage().contains("Connection is not created")
                        || ex.getMessage().startsWith("Failed to connect")) {
                    exception += "Failed to connect to " + readerName;
                } else {
                    exception += ex.getMessage();
                }
                LoggerUtil.error(TAG, "Exception while Connecting :", ex);
            }
            return exception;
        }

        @Override
        protected void onPostExecute(String exception) {
            pDialog.dismiss();

            if (!operationStatus) {
                AlertDialog deleteAlert = new AlertDialog.Builder(connectButton.getContext()).create();
                deleteAlert.setTitle("Connect");
                deleteAlert.setMessage("Connection Timeout");
                deleteAlert.show();
            }

            LoggerUtil.debug(TAG, "** ReaderConnectionThread.onPostExecute **");
            if (!operationStatus) {
                LoggerUtil.debug(TAG, "OperationStatus is disconnected");
                totalTagCountView.setText("");
                if (operation.equalsIgnoreCase("Connect")) {
                    String str = "Connect";
                    readButton.setClickable(false);
                    readButton.setEnabled(false);
                    readcontrollayout.setVisibility(View.GONE);
                    enableEdit();
                    totalTagCountView.setText("");
                    searchResultCount.setText("");
                    avgReadView.setText("");
                    readingTimeView.setText("");
                    temperatureView.setText("");
                    mReaderActivity.reader = null;
                    regionList.setEnabled(true);
                    System.gc();
                }
            } else {
                LoggerUtil.debug(TAG, "OperationStatus is connected");

                if (operation.equalsIgnoreCase("Connect")) {
                    String str = "Disconnect";
                    connectButton.setText(str);
                    readcontrollayout.setVisibility(View.VISIBLE);
                    connectButton.setEnabled(true);
                    connectButton.setClickable(true);
                    readButton.setClickable(true);
                    readButton.setEnabled(true);
                    mReaderActivity.reader = reader;
                    regionList.setEnabled(false);
                    powerButton.setEnabled(false);
                    baudrateList.setEnabled(false);

                    //int powerLevel = (int)mReaderActivity.reader.paramGet("/reader/radio/readPower");
                    int powerLevel = 2600;

                    powerList.setSelection((powerLevel / 100) - 10);

                    mReaderActivity.updateSettings();
                } else {
                    String str = "Connect";
                    layout_firmware.setVisibility(View.GONE);
                    firmware_hint.setVisibility(View.VISIBLE);
                    connectButton.setText(str);
                    powerButton.setEnabled(true);
                    readButton.setClickable(false);
                    readButton.setEnabled(false);
                    readcontrollayout.setVisibility(View.GONE);
                    if (!mReaderActivity.checkScanService.isChecked()) {
                        enableEdit();
                    }
                    mReaderActivity.enableBarcode();
                    totalTagCountView.setText("");
                    searchResultCount.setText("");
                    avgReadView.setText("");
                    readingTimeView.setText("");
                    temperatureView.setText("");
                    mReaderActivity.reader = null;
                    regionList.setEnabled(true);
                    baudrateList.setEnabled(true);
                    System.gc();

                    this.cancel(false);
                }
            }
        }

        private void disableEdit() {
            connectButton.setEnabled(false);
            connectButton.setClickable(false);
        }

        private void enableEdit() {
            connectButton.setEnabled(true);
            connectButton.setClickable(true);
        }

        private void timerDelayRemoveDialog(long time, final ProgressDialog d) {
            if (timeoutRunnable == null) {
                timeoutRunnable = new Runnable() {
                    public void run() {
                        Log.i(TAG, "Run timeout runnable");
                        if (d.isShowing()) {
                            d.dismiss();
                            operationStatus = false;
                            AlertDialog deleteAlert = new AlertDialog.Builder(connectButton.getContext()).create();
                            deleteAlert.setTitle("Connect");
                            deleteAlert.setMessage("Connection Timeout");
                            deleteAlert.show();
                            onPostExecute("timeout");
                        }
                    }
                };
            } else {
                Log.i(TAG, "removeCallbacks timeout runnable");
                mHandler.removeCallbacks(timeoutRunnable);
            }
            Log.i(TAG, "timerDelayRemoveDialog");
            mHandler.postDelayed(timeoutRunnable, time);
        }
    }
}
