package com.unitech.rfidreader;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.os.Bundle;
import android.text.InputType;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.WindowManager;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.ExpandableListView;
import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.ToggleButton;

import com.thingmagic.Reader;
import com.unitech.pa760pogoapi.PogoManager;
import com.unitech.rfidreader.Listener.ConnectionListener;
import com.unitech.rfidreader.Listener.FirmwareListener;
import com.unitech.rfidreader.Listener.ServiceListener;
import com.unitech.rfidreader.customViews.ExpandableListAdapter;
import com.unitech.util.Beeper;
import com.unitech.util.GlobalExceptionHandler;
import com.unitech.util.LoggerUtil;
import com.unitech.util.Utilities;

import java.io.File;
import java.util.ArrayList;

public class ReaderActivity extends Activity {

    private static final String TAG = "ReaderActivity";
    private static final int KEYCODE_SCAN_GUN = 286, KEYCODE_SCAN_EXTENDED = 290;
    private Button connectButton = null;
    private Button scanButton = null;
    private Button clearButton = null;
    private Button settingsButton = null;
    private Button homeButton = null;


    private Button btn_read;
    private Button btn_write;
    private DisplayMetrics displaymetrics = new DisplayMetrics();
    private TextView searchResultCount = null;
    private TextView rowNumberLabelView = null;
    private TextView epcRSSILabelView = null;
    private TextView epcDataLabelView = null;
    private TextView epcCountLabelView = null;

    private int windowWidth;
    public int rowNumberWidth = 0;
    public int epcRSSIWidth = 0;
    public int epcDataWidth = 0;
    public int epcCountWidth = 0;
    public Reader reader = null;
    public View readOptions;
    public View performance_metrics;
    public View firmware;
    public View about;
    private Spinner regionList = null;
    private Spinner powerList = null;


    private LinearLayout readcontrollayout = null;
    private LinearLayout read_result_layout = null;
    private View tableTags = null;

    public com.unitech.rfidreader.customViews.ExpandableListAdapter expandableListAdapter;

    public CheckBox checkBeep = null;
    public CheckBox checkScanService = null;
    public Switch powerSwitch;
    public ToggleButton powerButton;
    public EditText barcodeText = null;

    /**
     * The flag indicating whether content is loaded (text is shown) or not (setting view is
     * shown).
     */
    private boolean mContentLoaded;
    private View mSettingView;
    private View mDisplayView;

    private PogoManager mPogoManager = PogoManager.getInstance();
    private ServiceListener serviceListener;
    private FirmwareListener firmwareListener;

    private Beeper mBeeper;

    private boolean isCallingScanService = false;

    //==============================================================================================
    @Override
    public void onCreate(Bundle savedInstanceState) {
        try {
            super.onCreate(savedInstanceState);
            setContentView(R.layout.activity_reader);
            getWindow().setFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON, WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
            //-------------------------------------------------------------------------------------
            LoggerUtil.initialize(this);
            String path = String.format("%s/%s", this.getExternalFilesDir(null).getAbsolutePath(), LoggerUtil.DIR_LOG);
            File file = new File(path);
            if (!file.exists()) {
                if (file.mkdir()) {
                    LoggerUtil.debug(TAG, "Build AutoTestLog File. ");
                }
            }
            System.out.println(TAG);
            //--------------------------------------------------------------------------------------
            LoggerUtil.debug(TAG, "RFIDReader Version : " + getVersionName(this));
            mSettingView = findViewById(R.id.settings);
            mDisplayView = findViewById(R.id.displayView);

            // Initially hide the setting view.
            mSettingView.setVisibility(View.GONE);

            ReaderActivity activity = this;
            try {
                Utilities.loadLog4j(this);
            } catch (Exception ex) {
                LoggerUtil.error(TAG, "Error loading log4j", ex);
            }
            this.findAllViewsById();
            windowWidth = displaymetrics.widthPixels;

            loadSettingConntentView();

            connectButton.setOnClickListener(new ConnectionListener(this));

            searchResultCount.setText("");

            clearButton.setOnClickListener(new ServiceListener(this).clearListener);
            serviceListener = new ServiceListener(this);
            scanButton.setOnClickListener(serviceListener);

            btn_write.setOnClickListener(new ServiceListener(this).btn_RFID_write);
            btn_read.setOnClickListener(new ServiceListener(this).btn_RFID_read);
            settingsButton.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View view) {
                    mContentLoaded = !mContentLoaded;
                    showSettingsOrTagsView(mContentLoaded);
                }
            });

            // Create an ArrayAdapter using the string array and a default spinner layout
            ArrayAdapter<CharSequence> regionadapter = ArrayAdapter.createFromResource(this, R.array.regions_array, android.R.layout.simple_spinner_item);
            // Specify the layout to use when the list of choices appears
            regionadapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
            // Apply the adapter to the spinner
            regionList.setAdapter(regionadapter);
            regionList.setSelection(15);


            powerList = (Spinner) this.readOptions.findViewById(R.id.antennaPower);
            // Create an ArrayAdapter using the string array and a default spinner layout
            ArrayAdapter<CharSequence> poweradapter = ArrayAdapter.createFromResource(this, R.array.powerlevel_array, android.R.layout.simple_spinner_item);
            // Specify the layout to use when the list of choices appears
            poweradapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
            // Apply the adapter to the spinner
            powerList.setAdapter(poweradapter);
            powerList.setSelection(20);

            powerSwitch = (Switch) this.firmware.findViewById(R.id.switchPower);
            powerSwitch.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {
                    try {
                        if (powerSwitch.isChecked() && !mPogoManager.isPowerOn()) {
                            PA760RFIDModulePowerCtrl(true);
                        } else if (!powerSwitch.isChecked() && mPogoManager.isPowerOn()) {
                            PA760RFIDModulePowerCtrl(false);
                        }
                    } catch (PogoManager.AccessError accessError) {
                        accessError.printStackTrace();
                    }
                }
            });

            powerButton.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {
                    try {
                        if (powerButton.isChecked() && !mPogoManager.isPowerOn()) {
                            PA760RFIDModulePowerCtrl(true);
                        } else if (!powerButton.isChecked() && mPogoManager.isPowerOn()) {
                            PA760RFIDModulePowerCtrl(false);
                        }
                    } catch (PogoManager.AccessError accessError) {
                        accessError.printStackTrace();
                    }
                }
            });

            firmwareListener = new FirmwareListener(this);

            readcontrollayout.setVisibility(View.GONE);

            setPortraitTableLayout();
            System.gc();
            Thread.currentThread().setDefaultUncaughtExceptionHandler(new GlobalExceptionHandler(activity));
            PA760RFIDModulePowerCtrl(false);

        } catch (Exception ex) {
            ex.printStackTrace();
        }

        createBeeper();
//        PogoManager.getInstance().setScanSource(ReaderActivity.this, PogoManager.EXTERNAL_SCANNER);
//        setScanGunKey(true);

        registerScanServiceBroadcastReceiver();
    }

    @Override
    protected void onResume() {
        if (isCallingScanService) {
            isCallingScanService = false;
        } else {
            if (checkScanService.isChecked()) {
                stopScan2Key();
                setScanGunKey(false);
            } else {
                startScan2Key();
//                PogoManager.getInstance().setScanSource(ReaderActivity.this, PogoManager.INTERNAL_SCANNER);
                setScanGunKey(true);
            }
        }
        PA760RFIDModulePowerCtrl(true);

        super.onResume();
    }

    @Override
    protected void onPause() {
        if (firmwareListener.isFirmwareUpgrading()) {
            super.onPause();
            return;
        }

        if (serviceListener != null && serviceListener.isReading()) {
            serviceListener.stopReading();
            serviceListener.clearTagRecords();
        }

        if (!isCallingScanService) {
            if (connectButton.getText().equals("Disconnect")) {
                connectButton.performClick();
            }

            PA760RFIDModulePowerCtrl(false);

//            if (checkScanService.isChecked()) {
//                startScan2Key();
//            }

            setScanGunKey(false);
            isCallingScanService = false;
        }

//        isCallingScanService = true;
//        PogoManager.getInstance().setScanSource(ReaderActivity.this, PogoManager.EXTERNAL_SCANNER);
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        unregisterReceiver(scanServiceReceiver);
        startScan2Key();
//        PogoManager.getInstance().setScanSource(ReaderActivity.this, PogoManager.INTERNAL_SCANNER);
        setScanGunKey(false);
        super.onDestroy();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        switch (requestCode) {
            case FirmwareListener.FILE_SELECT_CODE:
                if (resultCode == RESULT_OK) {
                    firmwareListener.onFileSelectorResult(data);
                }
                break;
        }
        super.onActivityResult(requestCode, resultCode, data);
    }

    //==============================================================================================
    @SuppressLint("InflateParams")
    private void loadSettingConntentView() {
        ExpandableListView expandableList = (ExpandableListView) findViewById(R.id.expandableListView1);

        LayoutInflater infalInflater = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        assert infalInflater != null;
        readOptions = infalInflater.inflate(R.layout.settings_read_options, expandableList, false);
        firmware = infalInflater.inflate(R.layout.firmware, expandableList, false);
        about = infalInflater.inflate(R.layout.about, expandableList, false);
        ArrayList<View> ChildViews = new ArrayList<>();
        ChildViews.add(readOptions);
        ChildViews.add(firmware);
        ChildViews.add(about);

        String[] groups = {"Read Options", "Firmware", "About"};
        expandableListAdapter = new ExpandableListAdapter(this, groups, ChildViews);

        // Set this blank adapter to the list view
        expandableList.setAdapter(expandableListAdapter);
        homeButton.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View view) {
                mContentLoaded = !mContentLoaded;
                showSettingsOrTagsView(mContentLoaded);

                ServiceListener.saveReadOptions();

                if (connectButton != null
                        && connectButton.getText().equals(getString(R.string.Disconnect))
                        && scanButton.getText().equals(getString(R.string.start_reading))
                        && !firmwareListener.isFirmwareUpgrading()) {
                    ServiceListener.OpenRFIDPower();
                }
            }
        });
    }

    public void updateSettings() {
        ServiceListener.updateSettings();
//        ServiceListener.btn_default.performClick();
//        ServiceListener.ShowPreReadOptions();
        ServiceListener.OpenRFIDPower();
    }

    public void closeSettings() {
        mContentLoaded = !mContentLoaded;
        showSettingsOrTagsView(mContentLoaded);
    }

    /**
     * Cross-fades between {@link #mSettingView} and {@link #mDisplayView}.
     */
    private void showSettingsOrTagsView(boolean contentLoaded) {
        // Decide which view to hide and which to show.
        final View showView = contentLoaded ? mSettingView : mDisplayView;
        final View hideView = contentLoaded ? mDisplayView : mSettingView;

        showView.setVisibility(View.VISIBLE);
        hideView.setVisibility(View.GONE);
    }

    /**
     * This is called when the screen rotates.
     * (onCreate is no longer called when screen rotates due to manifest, see: android:configChanges)
     */
    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        int orientation = newConfig.orientation;
        if (orientation == Configuration.ORIENTATION_PORTRAIT) {
            setPortraitTableLayout();
        } else if (orientation == Configuration.ORIENTATION_LANDSCAPE) {
            setPortraitTableLayout();
        }
    }

    private void setPortraitTableLayout() {
        rowNumberWidth = (int) (windowWidth * (0.09));
        epcDataWidth = (int) (windowWidth * (0.61));
        epcRSSIWidth = (int) (windowWidth * (0.13));
        epcCountWidth = (int) (windowWidth * (0.17));
        rowNumberLabelView.setWidth(rowNumberWidth);
        epcDataLabelView.setWidth(epcDataWidth);
        epcRSSILabelView.setWidth(epcRSSIWidth);
        epcCountLabelView.setWidth(epcCountWidth);
    }

    @Override
    public void onBackPressed() {
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (checkScanService.isChecked()) {
            return super.onKeyDown(keyCode, event);
        }
        switch (keyCode) {
            default:
                return super.onKeyDown(keyCode, event);
            case KEYCODE_SCAN_EXTENDED:
                if (ReaderActivity.this.reader != null && connectButton.getText().equals("Disconnect")
                        && serviceListener != null) {
                    serviceListener.startReading();
                }
                return true;
        }
    }

    @Override
    public boolean onKeyLongPress(int keyCode, KeyEvent event) {
        if (checkScanService.isChecked()) {
            return super.onKeyLongPress(keyCode, event);
        }
        switch (keyCode) {
            default:
                return super.onKeyLongPress(keyCode, event);
            case KEYCODE_SCAN_EXTENDED:
                if (ReaderActivity.this.reader != null && connectButton.getText().equals("Disconnect")
                        && serviceListener != null && !serviceListener.isReading()) {
                    serviceListener.startReading();
                }
                return true;
        }
    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {
        if (checkScanService.isChecked()) {
            return super.onKeyDown(keyCode, event);
        }
        switch (keyCode) {
            default:
                return super.onKeyUp(keyCode, event);
            case KEYCODE_SCAN_EXTENDED:
                if (serviceListener != null && serviceListener.isReading()) {
                    serviceListener.stopReading();
                }
                return true;
        }
    }

    private void findAllViewsById() {
        getWindowManager().getDefaultDisplay().getMetrics(displaymetrics);

        connectButton = (Button) findViewById(R.id.btn_connect);
        scanButton = (Button) findViewById(R.id.btn_scan);
        clearButton = (Button) findViewById(R.id.btn_Clear);
        settingsButton = (Button) findViewById(R.id.btn_setting);
        homeButton = (Button) findViewById(R.id.btn_back_main);

        //------------------------------------------------------------------------------------------
        btn_write = (Button) findViewById(R.id.btn_write);
        btn_read = (Button) findViewById(R.id.btn_read);
        //------------------------------------------------------------------------------------------
        searchResultCount = (TextView) findViewById(R.id.search_result_view);
        rowNumberLabelView = (TextView) findViewById(R.id.SNOLabel);
        epcDataLabelView = (TextView) findViewById(R.id.EPCLabel);
        epcRSSILabelView = (TextView) findViewById(R.id.RSSILabel);
        epcCountLabelView = (TextView) findViewById(R.id.COUNTLabel);

        readcontrollayout = (LinearLayout) findViewById(R.id.ReadControlLayout);
        read_result_layout = findViewById(R.id.read_result_layout);

        regionList = (Spinner) findViewById(R.id.Region_List);
        checkBeep = (CheckBox) findViewById(R.id.BeepControl);
        checkBeep.setOnCheckedChangeListener(beeperCheckedListener);
        checkScanService = findViewById(R.id.BarcodeControl);
        checkScanService.setOnCheckedChangeListener(scanServiceCheckedListener);
        barcodeText = findViewById(R.id.barcodeText);
        barcodeText.setRawInputType(InputType.TYPE_NULL);

        powerButton = findViewById(R.id.btn_power);

        tableTags = findViewById(R.id.table_tags);
    }

    public void PA760RFIDModulePowerCtrl(final boolean bEnable) {

        try {
            if (bEnable) {
                mPogoManager.turnOnPower();
                if (powerSwitch != null) {
                    powerSwitch.setChecked(true);
                }
                if (powerButton != null) {
                    powerButton.setChecked(true);
                }
            } else {
                mPogoManager.turnOffPower();
                if (powerSwitch != null) {
                    powerSwitch.setChecked(false);
                }
                if (powerButton != null) {
                    powerButton.setChecked(false);
                }
            }
        } catch (Exception ex) {
            LoggerUtil.error("RFID Power Control", "Get exception!");
            connectButton.setText(getResources().getString(R.string.exception));
            ex.printStackTrace();
        }
    }

    private void createBeeper() {
        mBeeper = Beeper.getInstance();
        mBeeper.setCanBeep(checkBeep.isChecked());
    }

    private CheckBox.OnCheckedChangeListener beeperCheckedListener = new CheckBox.OnCheckedChangeListener() {
        @Override
        public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
            if (isChecked) {
                mBeeper.setCanBeep(true);
                mBeeper.stopAllBeep();
                mBeeper.tryBeep();
            } else {
                mBeeper.stopAllBeep();
                mBeeper.setCanBeep(false);
            }
        }
    };

    private CheckBox.OnCheckedChangeListener scanServiceCheckedListener = new CheckBox.OnCheckedChangeListener() {
        @Override
        public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
            if (isChecked) {
                barcodeText.setEnabled(true);
                barcodeText.setVisibility(View.VISIBLE);
                barcodeText.requestFocus();
                barcodeText.setShowSoftInputOnFocus(false);
                connectButton.setEnabled(false);

                if (scanButton.getText().equals(getString(R.string.start_reading)) && scanButton.isEnabled()) {
                    scanButton.setEnabled(false);
                    scanButton.setTag(Boolean.TRUE);
                }

                tableTags.setVisibility(View.GONE);
                if (readcontrollayout.getVisibility() == View.VISIBLE) {
                    readcontrollayout.setVisibility(View.GONE);
                    readcontrollayout.setTag(Boolean.TRUE);
                }

                if (read_result_layout.getVisibility() == View.VISIBLE) {
                    read_result_layout.setVisibility(View.GONE);
                    read_result_layout.setTag(Boolean.TRUE);
                }

//                isCallingScanService = true;
//                PogoManager.getInstance().setScanSource(ReaderActivity.this, PogoManager.INTERNAL_SCANNER);
                setScanGunKey(false);
            } else {
                barcodeText.setEnabled(false);
                barcodeText.setVisibility(View.GONE);
                connectButton.setEnabled(true);
                connectButton.setClickable(true);
                if (connectButton.getText().equals(getString(R.string.Connect))) {
                    scanButton.setEnabled(false);
                }
                if (scanButton.getTag() == Boolean.TRUE) {
                    scanButton.setTag(Boolean.FALSE);
                    scanButton.setEnabled(true);
                }
                tableTags.setVisibility(View.VISIBLE);
                if (readcontrollayout.getTag() == Boolean.TRUE) {
                    readcontrollayout.setTag(Boolean.FALSE);
                    readcontrollayout.setVisibility(View.VISIBLE);
                }

                if (read_result_layout.getTag() == Boolean.TRUE) {
                    read_result_layout.setTag(Boolean.FALSE);
                    read_result_layout.setVisibility(View.VISIBLE);
                }

//                isCallingScanService = true;
//                PogoManager.getInstance().setScanSource(ReaderActivity.this, PogoManager.EXTERNAL_SCANNER);
                setScanGunKey(true);
            }
        }
    };

    public void clearBarcodeText() {
        this.barcodeText.getText().clear();
    }

    public void disableBarcode() {
        this.checkScanService.setEnabled(false);
    }

    public void enableBarcode() {
        this.checkScanService.setEnabled(true);
    }

    private void registerScanServiceBroadcastReceiver() {
        IntentFilter scannerIntentFilter = new IntentFilter();
        scannerIntentFilter.addAction("unitech.scanservice.data");
        scannerIntentFilter.addAction("unitech.scanservice.databytelength");
        registerReceiver(scanServiceReceiver, scannerIntentFilter);
    }

    private void stopScan2Key() {
//        Intent intent = new Intent("unitech.scanservice.scan2key_setting");
//        intent.putExtra("scan2key", false);
//        sendBroadcast(intent);
    }

    private void startScan2Key() {
//        Intent intent = new Intent("unitech.scanservice.scan2key_setting");
//        intent.putExtra("scan2key", true);
//        sendBroadcast(intent);
    }

    /**
     * Set the key code of SCAN_GUN
     *
     * @param isExtended <br/>true: set KeyCode to SCAN_EXTENDED <br/>false: set KeyCode to SCAN_GUN
     */
    private void setScanGunKey(boolean isExtended) {
        isCallingScanService = true;
        if (isExtended) {
            if (PogoManager.getInstance().getScanGunSource(ReaderActivity.this) != PogoManager.EXTERNAL_SCANNER) {
                Log.d(TAG, "Set scan gun key to extended");
                PogoManager.getInstance().setScanSource(ReaderActivity.this, PogoManager.EXTERNAL_SCANNER);
            }
        } else {
            if (PogoManager.getInstance().getScanGunSource(ReaderActivity.this) != PogoManager.INTERNAL_SCANNER) {
                Log.d(TAG, "Set scan gun key to scan_gun");
                PogoManager.getInstance().setScanSource(ReaderActivity.this, PogoManager.INTERNAL_SCANNER);
            }
        }
    }

    public String getVersionName(Context context) {
        PackageManager packageManager = context.getPackageManager();
        PackageInfo packageInfo;
        String versionName = "";
        try {
            packageInfo = packageManager.getPackageInfo(context.getPackageName(), 0);
            versionName = packageInfo.versionName;
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
        return versionName;
    }

    private BroadcastReceiver scanServiceReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            Bundle bundle;
            if (intent != null) {
                switch (intent.getAction()) {
                    case "unitech.scanservice.data":
                        //收到條碼資料字串
                        bundle = intent.getExtras();
                        if (bundle != null) {
                            String barcodeData = bundle.getString("text");//取得資料字串
                            Log.d(TAG, String.format("Recv scan str: %s", barcodeData));
                            barcodeText.append(barcodeData);
                        }
                        break;
                    case "unitech.scanservice.databytelength":
                        //收到條碼資料長度
                        bundle = intent.getExtras();
                        if (bundle != null) {
                            int len = bundle.getInt("text");//取得資料長度
                            Log.d(TAG, String.format("Recv scan len: %d", len));
                        }
                        break;
                }
            }
        }
    };
}