package com.unitech.util;

import android.media.AudioManager;
import android.media.ToneGenerator;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.locks.ReentrantLock;

/**
 * Created by USER
 */
public class Beeper {

    private static Beeper mInstance;

    private final ToneGenerator toneGen;
    private final ExecutorService executorService;
    private ReentrantLock beeperLock, beepingLock;
    private boolean canBeep = false, toBeep = false;

    public static Beeper getInstance() {
        if (mInstance == null) {
            mInstance = new Beeper();
        }

        return mInstance;
    }

    private Beeper() {
        this.toneGen = new ToneGenerator(AudioManager.STREAM_MUSIC, 100);
        this.beeperLock = new ReentrantLock();
        this.beepingLock = new ReentrantLock();
        this.executorService = Executors.newSingleThreadExecutor();
    }

    public void setCanBeep(boolean canBeep) {
        this.canBeep = canBeep;
    }

    public void beep() {
        if (canBeep && !toBeep && beeperLock.tryLock()) {
            toBeep = true;
            executorService.submit(runBeep);
            beeperLock.unlock();
        }
    }

    public void tryBeep() {
        beepingLock.lock();
        toneGen.startTone(ToneGenerator.TONE_CDMA_LOW_L, 25);
        beepingLock.unlock();
    }

    public void stopAllBeep() {
        beeperLock.lock();
        toBeep = false;
        beeperLock.unlock();
    }

    private Runnable runBeep = new Runnable() {
        @Override
        public void run() {
            if (beepingLock.tryLock() && toBeep && canBeep) {
                toneGen.startTone(ToneGenerator.TONE_CDMA_LOW_L, 25);
                try {
                    Thread.sleep(50);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                toBeep = false;
                beepingLock.unlock();
            }
        }
    };
}
