package com.unitech.util;

import android.content.Context;
import android.content.SharedPreferences;

public class Configuration {

    private final String Custom_Conf = "Custom";

    private final String Key_Operation_Mode = "OperationMode";
    private final String Key_Reader_Timeout = "ReaderTimeout";
    private final String Key_Antenna_On_Time = "OnTime";
    private final String Key_Antenna_Off_Time = "OffTime";
    private final String Key_Session = "Session";
    private final String Key_Target = "Target";
    private final String Key_Fast_Search = "FastSearch";
    private final String Key_Q_Value = "QValue";
    private final String Key_Encoding = "Encoding";
    private final String Key_TARI = "TARI";
    private final String Key_BLF ="BLF";
    private final String Key_Power = "Power";

    private SharedPreferences properties;

    private static Context appContext;

    public Configuration(Context context) {
        appContext = context;
        loadConf();
    }

    private void setDefault() {

        setOperationMode(1);

        setReaderTimeout(1000);

        setAntennaOnTime(100);

        setAntennaOffTime(100);

        setPower(20);

        setBLF(0);

        setTARI(0);

        setSession(1);

        setEncoding(3);

        setQValue(0);

        setTarget(2);

        setFastSearch(false);
    }

    public void loadConf() {

        properties = appContext.getApplicationContext().getSharedPreferences(Custom_Conf, Context.MODE_PRIVATE);

        if (!properties.contains(Key_Operation_Mode))
            setOperationMode(1);

        if (!properties.contains(Key_Reader_Timeout))
            setReaderTimeout(1000);

        if (!properties.contains(Key_Antenna_On_Time))
            setAntennaOnTime(100);

        if (!properties.contains(Key_Antenna_Off_Time))
            setAntennaOffTime(100);

        if (!properties.contains(Key_Power))
            setPower(20);

        if (!properties.contains(Key_BLF))
            setBLF(0);

        if (!properties.contains(Key_TARI))
            setTARI(0);

        if (!properties.contains(Key_Session))
            setSession(1);

        if (!properties.contains(Key_Encoding))
            setEncoding(3);

        if (!properties.contains(Key_Q_Value))
            setQValue(0);

        if (!properties.contains(Key_Target))
            setTarget(2);

        if (!properties.contains(Key_Fast_Search))
            setFastSearch(false);
    }

    public void setOperationMode(int mode) {
        properties.edit().putInt(Key_Operation_Mode, mode).commit();
    }

    public int getOperationMode() {
        if (!properties.contains(Key_Operation_Mode))
            setOperationMode(1);

        return properties.getInt(Key_Operation_Mode, 1);
    }

    public void setReaderTimeout(int timeout) {
        properties.edit().putInt(Key_Reader_Timeout, timeout).commit();
    }

    public int getReaderTimeout() {
        if (!properties.contains(Key_Reader_Timeout))
            setReaderTimeout(1000);

        return properties.getInt(Key_Reader_Timeout, 1000);
    }

    public void setAntennaOnTime(int onTime) {
        properties.edit().putInt(Key_Antenna_On_Time, onTime).commit();
    }

    public int getAntennaOnTime() {
        if (!properties.contains(Key_Antenna_On_Time))
            setAntennaOnTime(100);

        return properties.getInt(Key_Antenna_On_Time, 100);
    }

    public void setAntennaOffTime(int offTime) {
        properties.edit().putInt(Key_Antenna_Off_Time, offTime).commit();
    }

    public int getAntennaOffTime() {
        if (!properties.contains(Key_Antenna_Off_Time))
            setAntennaOffTime(100);

        return properties.getInt(Key_Antenna_Off_Time, 100);
    }

    public void setPower(int power) {
        properties.edit().putInt(Key_Power, power).commit();
    }

    public int getPower() {
        if (!properties.contains(Key_Power))
            setPower(20);

        return properties.getInt(Key_Power, 20);
    }

    public void setSession(int session) {
        properties.edit().putInt(Key_Session, session).commit();
    }

    public int getSession() {
        if (!properties.contains(Key_Session))
            setSession(1);

        return properties.getInt(Key_Session, 1);
    }

    public void setTarget(int target)
    {
        properties.edit().putInt(Key_Target, target).commit();
    }

    public int getTarget()
    {
        if (!properties.contains(Key_Target))
            setTarget(2);

        return properties.getInt(Key_Target, 2);
    }

    public void setBLF(int blf){
        properties.edit().putInt(Key_BLF, blf).commit();
    }

    public int getBLF(){
        if (!properties.contains(Key_BLF))
            setBLF(0);

        return properties.getInt(Key_BLF, 0);
    }

    public void setTARI(int tari){
        properties.edit().putInt(Key_TARI, tari).commit();
    }

    public int getTARI(){
        if (!properties.contains(Key_TARI))
            setTARI(0);

        return properties.getInt(Key_TARI, 0);
    }

    public void setEncoding(int encoding){
        properties.edit().putInt(Key_Encoding, encoding).commit();
    }

    public int getEncoding(){
        if (!properties.contains(Key_Encoding))
            setEncoding(3);

        return properties.getInt(Key_Encoding, 3);
    }

    public void setQValue(int q){
        properties.edit().putInt(Key_Q_Value, q).commit();
    }

    public int getQValue(){
        if (!properties.contains(Key_Q_Value))
            setQValue(0);

        return properties.getInt(Key_Q_Value, 0);
    }

    public void setFastSearch(boolean fastSearch)
    {
        properties.edit().putBoolean(Key_Fast_Search, fastSearch).commit();
    }

    public boolean getFastSearch(){
        if (!properties.contains(Key_Fast_Search))
            setFastSearch(false);

        return properties.getBoolean(Key_Fast_Search, false);
    }
}
